﻿using System;
using System.Windows.Forms;
using System.Diagnostics;
using System.Threading;
using org.mbed.RPC;

namespace mbedRPC.Test
{
    public partial class mbedRCPTest : Form
    {
        // the mebed RPC methods
        private DigitalOut  led1;           // mbed DigitalOut RPC declaration -> LED1
        private DigitalOut led2;            // mbed DigitalOut RPC declaration -> LED2
        private DigitalOut led3;            // mbed DigitalOut RPC declaration -> LED1
        private DigitalOut led4;            // mbed DigitalOut RPC declaration -> LED2
        private AnalogOut analogOut;
        private AnalogIn analogIn;
        private PwmOut pwmOut;

        // the mbed objects 
        private HTTPRPC mbedHTTP;
        private mbed mbed;                  // the embed itself

        public mbedRCPTest()
        {
            InitializeComponent();
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            try
            {
                mbed.delete();
                Debug.Print("Complete");
            }
            catch (NullReferenceException ex)
            {
               Debug.Print("No Reference: " + ex.Message);
            }
            this.Close();
        }

        private void mbedRCPTest_Load(object sender, EventArgs e)
        {
            // !!!!!ATTENTION!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            // Set your mbed WEB Server address here!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            // This is the address you'll get back in your terminal programme !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
            String myMBEDAddress = "http://192.168.2.108";      // this is the address the router gave me back
            Debug.Print("mbed RPC Hello World");

		    //Create an mbed object for communication over USB (serial)
		    //mbed = new SerialRxTxRPC("COM10", 9600);
            //Or over serial: 
            //mbed = new SerialRPC("COM10", 9600);
		    //Or over HTTP:  
            //mbed = new HTTPRPC("http://192.168.2.2")

            // but test if reachable first
            mbedHTTP = new HTTPRPC(myMBEDAddress);

            if (mbedHTTP.IsAlive == false)
            {
                // not reachable -> wrong ip address or not connected, etc.
                labelAnswer.Text = mbedHTTP.StatusMessage;
            }
            else
            {
                labelAnswer.Text = mbedHTTP.StatusMessage;

                //both ways are possible
                //Decide with is okay for you
                mbed = mbedHTTP.MBED;                       // I prefer this one!
                //or:
                //mbed = new HTTPRPC(myMBEDAddress);        
            }

            try
            {
                //declare in/outputs on the mbed
                led1 = new DigitalOut(mbed, mbed.LED1);
                led2 = new DigitalOut(mbed, mbed.LED2);
                led3 = new DigitalOut(mbed, mbed.LED3);
                led4 = new DigitalOut(mbed, mbed.LED4);
                analogOut = new AnalogOut(mbed, mbed.p18);
                analogIn = new AnalogIn(mbed, mbed.p19);
                pwmOut = new PwmOut(mbed, mbed.LED4);
            }
            catch (NullReferenceException ex)
            {
                Debug.Print("No Reference at loading test: " + ex.Message);
            }
            catch (Exception ex)
            {
                //something went wrong
                Debug.Print("Error at loading test: " + ex.Message);
            }
        }

        private void flash(int n)
        {
            try
            {
                for (int i = 0; i < n; i++)
                {
                    //Call the DigitalOut objects' methods 
                    led2.write(1);          // on
                    led3.write(0);          // off
                    Thread.Sleep(250);      
                    led2.write(0);          // off
                    led3.write(1);          // on
                    Thread.Sleep(250);
                }
                led3.write(0);
                led2.write(0);        // reset last state - all leds are off now
            }
            catch (NullReferenceException ex)
            {
                Debug.Print("No Reference: " + ex.Message);
            }
        }

        private void btnFlash_Click(object sender, EventArgs e)
        {
            btnFlash.Enabled = false;
            btnExit.Enabled = false;
            btnFlash.Text = "Is Flashing";
            flash(5);
            btnFlash.Text = "Flashing LED2+3";
            btnExit.Enabled = true;
            btnFlash.Enabled = true;
        }

        private void checkBoxLED1_CheckedChanged(object sender, EventArgs e)
        {
            if(led1 == null) return;

            if (checkBoxLED1.CheckState == CheckState.Checked)
                led1.write(1);
            else
                led1.write(0);
        }

        private void checkBoxLED2_CheckedChanged(object sender, EventArgs e)
        {
            if (led2 == null) return;

            if (checkBoxLED2.CheckState == CheckState.Checked)
                led2.write(1);
            else
                led2.write(0);
        }

        private void checkBoxLED3_CheckedChanged(object sender, EventArgs e)
        {
            if (led3 == null) return;

            if (checkBoxLED3.CheckState == CheckState.Checked)
                led3.write(1);
            else
                led3.write(0);
        }

        private void btnAnalog_Click(object sender, EventArgs e)
        {
            analogOut.write(0.75f);
            double outVal = analogOut.read();
            double inVal = analogIn.read();
        }

        private void trackBarAnalogOut_ValueChanged(object sender, EventArgs e)
        {
            float val = trackBarAnalogOut.Value;
            float valOut = val / 10.0f;
            analogOut.write(valOut);

            double inVal = analogIn.read() * 3.3f;
            textBoxVoltage.Text = String.Format("{0:0.000}", Math.Round(inVal, 3));
        }

        private void btnPWM_Click(object sender, EventArgs e)
        {
            try
            {
                for (float p = 0.0f; p < 0.6f; p += 0.1f)
                {
                    pwmOut.write(p);
                }
                for (float p = 0.7f; p >= 0.0f; p -= 0.1f)
                {
                    pwmOut.write(p);
                }
                pwmOut.write(0.0f);
            }
            catch (NullReferenceException ex)
            {
                Debug.Print("No Reference: " + ex.Message);
            }
        }

        private void timerRPC_Tick(object sender, EventArgs e)
        {
            if (mbedHTTP != null)
            {
                if (mbedHTTP.IsAlive)
                {
                    btnReset.Enabled = false;
                    labelAnswer.Text = mbedHTTP.StatusMessage;
                }
                else
                {
                    btnReset.Enabled = true;
                    labelAnswer.Text = mbedHTTP.StatusMessage;
                }
            }
        }

        private void btnReset_Click(object sender, EventArgs e)
        {
            //check connection, router or cable
            //reset the flag and try again
            mbedHTTP.IsAlive = true;
        }
    }
}
